@@ -2,7 +2,7 @@ use std::fs;
 use tokio::time::{sleep, Duration};
 use tokio_modbus::prelude::*;
 
-use crate::{config::*, types::*, util::*};
+use crate::{config::*, data::*, types::*, util::*};
 
 pub async fn start_modbus_client(
     mut config: ModbusDeviceConfig,
@@ -17,8 +17,13 @@ pub async fn start_modbus_client(
         .expect("Client config missing")
         .requests;
     let mut counter: u16 = 0;
+    let endianness = config.common.endianness;
+
     for mut request in client_requests {
-        let server_id = request.server_id.take().expect("server id missing");
+        let server_id = request
+            .server_id
+            .take()
+            .expect("server id missing");
         let server = Slave(server_id);
         let mut ctx = match config.common.protocol_type {
             ProtocolType::TCP => {
@@ -49,17 +54,23 @@ pub async fn start_modbus_client(
             rlist.push(r);
         }
 
-        let mut section_repeat_times = request.repeat_times.or_else(|| Some(1)).unwrap();
+        let mut section_repeat_times = request
+            .repeat_times
+            .or_else(|| Some(1))
+            .unwrap();
         #[allow(unused_parens)]
         let section_indefinite_loop = (section_repeat_times == REPEAT_TIME_INDEFINITE);
         while section_repeat_times > 0 {
             if !section_indefinite_loop {
                 section_repeat_times -= 1;
             }
-            for r in &rlist {
+            for r in &mut rlist {
                 let start_addr = r.access_start_address;
                 let count = r.access_quantity;
-                let mut file_repeat_times = r.repeat_times.or_else(|| Some(1)).unwrap();
+                let mut file_repeat_times = r
+                    .repeat_times
+                    .or_else(|| Some(1))
+                    .unwrap();
                 #[allow(unused_parens)]
                 let file_indefinite_loop = (file_repeat_times == REPEAT_TIME_INDEFINITE);
                 let delay_in_100ms = r.delay.or_else(|| Some(0)).unwrap();
@@ -83,7 +94,8 @@ pub async fn start_modbus_client(
                                 config.verbose_mode,
                             );
                             ModbusRequestReturnType::ResultWithU16Vec(
-                                ctx.read_input_registers(start_addr, count).await,
+                                ctx.read_input_registers(start_addr, count)
+                                    .await,
                             )
                         }
                         FunctionCode::ReadHoldingRegisters => {
@@ -95,24 +107,27 @@ pub async fn start_modbus_client(
                                 config.verbose_mode,
                             );
                             ModbusRequestReturnType::ResultWithU16Vec(
-                                ctx.read_holding_registers(start_addr, count).await,
+                                ctx.read_holding_registers(start_addr, count)
+                                    .await,
                             )
                         }
                         FunctionCode::WriteMultipleRegisters => {
-                            let new_values =
-                                r.new_values.as_ref().expect("missing value for write");
-                            let data_type =
-                                r.data_type.as_ref().expect("missing data type for write");
+                            let new_values = r
+                                .new_values
+                                .take()
+                                .expect("missing value for write");
                             let mut data = Vec::<u16>::new();
-                            match data_type {
-                                DataType::Float32 => {
-                                    for v in new_values {
-                                        if let Ok(f) = v.parse::<f32>() {
-                                            data.extend(write_be_f32_into_u16(f));
-                                        }
-                                    }
-                                },
-                                _ => todo!(),
+                            for v in new_values {
+                                let d = ModbusRegisterData {
+                                    data_description: "".to_string(),
+                                    data_model_type: DataModelType::HoldingOrInputRegister,
+                                    data_access_type: None,
+                                    data_type: r
+                                        .data_type
+                                        .expect("missing data type for write"),
+                                    data_value: v,
+                                };
+                                d.write_into_u16(&mut data, endianness);
                             }
                             vprintln(
                                 &format!(
@@ -123,35 +138,79 @@ pub async fn start_modbus_client(
                             );
                             vprintln(&format!("{:?}", &data), config.verbose_mode);
                             ModbusRequestReturnType::ResultWithNothing(
-                                ctx.write_multiple_registers(start_addr, &data).await,
+                                ctx.write_multiple_registers(start_addr, &data)
+                                    .await,
                             )
                         }
                         FunctionCode::WriteSingleRegister => {
-                            let new_values =
-                                r.new_values.as_ref().expect("missing value for write");
-                            let data_type =
-                                r.data_type.as_ref().expect("missing data type for write");
-                            let data =
-                                match data_type {
-                                    DataType::Uint16 => parse_int::parse::<u16>(&new_values[0])
-                                        .expect("incorrect value for Uint16"),
-                                    _ => todo!(),
+                            let new_values = r
+                                .new_values
+                                .take()
+                                .expect("missing value for write");
+                            let mut data = Vec::<u16>::new();
+                            for v in new_values {
+                                let d = ModbusRegisterData {
+                                    data_description: "".to_string(),
+                                    data_model_type: DataModelType::HoldingOrInputRegister,
+                                    data_access_type: None,
+                                    data_type: r
+                                        .data_type
+                                        .expect("missing data type for write"),
+                                    data_value: v,
                                 };
+                                d.write_into_u16(&mut data, endianness);
+                            }
                             vprintln(
-                                &format!(
-                                    "writing register at {} with value:",
-                                    start_addr
-                                ),
+                                &format!("writing register at {} with value:", start_addr),
                                 config.verbose_mode,
                             );
                             vprintln(&format!("{:?}", data), config.verbose_mode);
                             ModbusRequestReturnType::ResultWithNothing(
-                                ctx.write_single_register(start_addr, data).await,
+                                ctx.write_single_register(start_addr, data[0])
+                                    .await,
+                            )
+                        }
+                        FunctionCode::ReadWriteMultipleRegisters => {
+                            let new_values = r
+                                .new_values
+                                .take()
+                                .expect("missing value for write");
+                            let mut data = Vec::<u16>::new();
+                            for v in new_values {
+                                let d = ModbusRegisterData {
+                                    data_description: "".to_string(),
+                                    data_model_type: DataModelType::HoldingOrInputRegister,
+                                    data_access_type: None,
+                                    data_type: r
+                                        .data_type
+                                        .expect("missing data type for write"),
+                                    data_value: v,
+                                };
+                                d.write_into_u16(&mut data, endianness);
+                            }
+                            vprintln(
+                                &format!(
+                                    "writing and read registers starting at {} with values:",
+                                    start_addr
+                                ),
+                                config.verbose_mode,
+                            );
+                            vprintln(&format!("{:?}", &data), config.verbose_mode);
+                            ModbusRequestReturnType::ResultWithU16Vec(
+                                ctx.read_write_multiple_registers(
+                                    start_addr,
+                                    data.len() as u16,
+                                    start_addr,
+                                    &data,
+                                )
+                                .await,
                             )
                         }
                         FunctionCode::WriteMultipleCoils => {
-                            let new_values =
-                                r.new_values.as_ref().expect("missing value for write");
+                            let new_values = r
+                                .new_values
+                                .as_ref()
+                                .expect("missing value for write");
                             let mut data = Vec::<bool>::new();
                             for v in new_values {
                                 if let Ok(f) = v.parse::<bool>() {
@@ -159,15 +218,13 @@ pub async fn start_modbus_client(
                                 }
                             }
                             vprintln(
-                                &format!(
-                                    "writing registers starting at {} with values:",
-                                    start_addr
-                                ),
+                                &format!("writing coils starting at {} with values:", start_addr),
                                 config.verbose_mode,
                             );
                             vprintln(&format!("{:?}", &data), config.verbose_mode);
                             ModbusRequestReturnType::ResultWithNothing(
-                                ctx.write_multiple_coils(start_addr, &data).await,
+                                ctx.write_multiple_coils(start_addr, &data)
+                                    .await,
                             )
                         }
                         FunctionCode::ReadCoils => {
@@ -185,12 +242,15 @@ pub async fn start_modbus_client(
                                 config.verbose_mode,
                             );
                             ModbusRequestReturnType::ResultWithBoolVec(
-                                ctx.read_discrete_inputs(start_addr, count).await,
+                                ctx.read_discrete_inputs(start_addr, count)
+                                    .await,
                             )
                         }
                         FunctionCode::WriteSingleCoil => {
-                            let new_values =
-                                r.new_values.as_ref().expect("missing value for write");
+                            let new_values = r
+                                .new_values
+                                .as_ref()
+                                .expect("missing value for write");
                             let data = new_values[0]
                                 .parse::<bool>()
                                 .expect("incorrect value for bool");
@@ -200,7 +260,8 @@ pub async fn start_modbus_client(
                             );
                             vprintln(&format!("{:?}", &data), config.verbose_mode);
                             ModbusRequestReturnType::ResultWithNothing(
-                                ctx.write_single_coil(start_addr, data).await,
+                                ctx.write_single_coil(start_addr, data)
+                                    .await,
                             )
                         }
                         _ => todo!(),
@@ -208,18 +269,20 @@ pub async fn start_modbus_client(
                     println!("{}", r.description);
                     match response {
                         ModbusRequestReturnType::ResultWithU16Vec(Ok(response)) => {
-                            let data_type =
-                                r.data_type.as_ref().expect("missing data type for write");
+                            let data_type = r
+                                .data_type
+                                .as_ref()
+                                .expect("missing data type for write");
                             match data_type {
                                 DataType::Float32 => println!(
                                     "===> {:?}",
-                                    write_be_u16_into_f32(response.as_slice())
+                                    write_u16_into_f32(response.as_slice(), endianness)
                                 ),
                                 DataType::Float64 => {
-                                    println!("===> {:?}", write_be_u16_into_f64(&response))
+                                    println!("===> {:?}", write_u16_into_f64(&response, endianness))
                                 }
                                 DataType::Uint32 => {
-                                    let data = response[0] as u32 | (response[1] as u32) << 16;
+                                    let data = write_u16_into_u32(&response, endianness);
                                     println!("===> {:?} ({:#010X})", data, data);
                                 }
                                 DataType::Uint16 => {
