@@ -1,9 +1,9 @@
 use crate::{types::*, util::*};
 use anyhow;
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 use std::{collections::HashMap, fmt::Write as FmtWrite};
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ModbusRegisterData {
     /// the data description
     pub data_description: String,
@@ -128,6 +128,7 @@ impl ModbusRegisterData {
                 if function_code == FunctionCode::WriteMultipleRegisters
                     || function_code == FunctionCode::WriteSingleRegister
                     || function_code == FunctionCode::ReadHoldingRegisters
+                    || function_code == FunctionCode::ReadWriteMultipleRegisters
                 {
                     true
                 } else {
@@ -139,6 +140,7 @@ impl ModbusRegisterData {
                     || function_code == FunctionCode::WriteSingleRegister
                     || function_code == FunctionCode::ReadInputRegisters
                     || function_code == FunctionCode::ReadHoldingRegisters
+                    || function_code == FunctionCode::ReadWriteMultipleRegisters
                 {
                     true
                 } else {
@@ -150,18 +152,18 @@ impl ModbusRegisterData {
         }
     }
 
-    pub fn write_to_be_u16(&self, registers: &mut Vec<u16>) -> usize {
+    pub fn write_into_u16(&self, registers: &mut Vec<u16>, e: EndiannessType) -> usize {
         match &self.data_type {
             DataType::Float32 => {
                 if let Ok(value) = self.data_value.parse::<f32>() {
-                    let tmp = write_be_f32_into_u16(value);
+                    let tmp = write_f32_into_u16(value, e);
                     registers.extend(tmp);
                     return 2;
                 }
             }
             DataType::Float64 => {
                 if let Ok(value) = self.data_value.parse::<f64>() {
-                    let tmp = write_be_f64_into_u16(value);
+                    let tmp = write_f64_into_u16(value, e);
                     registers.extend(tmp);
                     return 4;
                 }
@@ -184,9 +186,10 @@ impl ModbusRegisterData {
         return 0;
     }
 
-    pub fn read_from_be_u16(
+    pub fn read_from_u16(
         &mut self,
         it: &mut std::iter::Peekable<std::slice::Iter<u16>>,
+        e: EndiannessType
     ) -> usize {
         match &self.data_type {
             DataType::Float32 => {
@@ -198,7 +201,7 @@ impl ModbusRegisterData {
                         return 0;
                     }
                 }
-                self.data_value = write_be_u16_into_f32(&tmp).to_string();
+                self.data_value = write_u16_into_f32(&tmp, e).to_string();
                 return 2;
             }
             DataType::Float64 => {
@@ -210,7 +213,7 @@ impl ModbusRegisterData {
                         return 0;
                     }
                 }
-                self.data_value = write_be_u16_into_f64(&tmp).to_string();
+                self.data_value = write_u16_into_f64(&tmp, e).to_string();
                 return 4;
             }
             DataType::Uint32 => {
@@ -226,19 +229,19 @@ impl ModbusRegisterData {
                 return 2;
             }
             DataType::Uint16 => {
-                    if let Some(data) = it.next() {
-                        self.data_value = data.to_string();
-                        return 1;
-                    } else {
-                        return 0;
-                    }
+                if let Some(data) = it.next() {
+                    self.data_value = data.to_string();
+                    return 1;
+                } else {
+                    return 0;
+                }
             }
             _ => todo!(),
         }
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Deserialize, Serialize)]
 pub struct ModbusRegisterDatabase {
     db: HashMap<u16, ModbusRegisterData>,
 }
@@ -249,13 +252,14 @@ impl ModbusRegisterDatabase {
         register_addr: u16,
         values: Vec<u16>,
         function_code: FunctionCode,
+        e: EndiannessType,
     ) -> anyhow::Result<usize, ModbusExceptionCode> {
         let mut value_it = values.iter().peekable();
         let mut total_updated = 0_usize;
         let mut addr = register_addr;
         while let Some(data) = self.db.get_mut(&addr) {
             if data.is_function_code_supported(function_code) {
-                let registers_updated = data.read_from_be_u16(&mut value_it);
+                let registers_updated = data.read_from_u16(&mut value_it, e);
                 if registers_updated != 0 {
                     addr += registers_updated as u16;
                     total_updated += registers_updated;
@@ -277,6 +281,7 @@ impl ModbusRegisterDatabase {
         register_addr: u16,
         registers_to_write: u16,
         function_code: FunctionCode,
+        e: EndiannessType,
     ) -> anyhow::Result<Vec<u16>, ModbusExceptionCode> {
         let mut registers = Vec::<u16>::new();
         let mut count = registers_to_write as usize;
@@ -284,7 +289,7 @@ impl ModbusRegisterDatabase {
         let mut printout = String::new();
         while let Some(data) = self.db.get(&addr) {
             if data.is_function_code_supported(function_code) {
-                let registers_written = data.write_to_be_u16(&mut registers);
+                let registers_written = data.write_into_u16(&mut registers, e);
                 writeln!(&mut printout, "{}", data.data_description).unwrap();
                 writeln!(&mut printout, "{} ===>", data.data_value).unwrap();
                 if count >= registers_written {
@@ -305,25 +310,25 @@ impl ModbusRegisterDatabase {
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct IndependentCoil {
     value: bool,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct RegisterBitCoil {
     register: u16,
     bit: u16,
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 #[serde(tag = "type")]
 pub enum ModbusCoilDataValueType {
     Independent(IndependentCoil),
     RegisterBit(RegisterBitCoil),
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ModbusCoilData {
     /// the data description
     pub data_description: String,
@@ -333,6 +338,8 @@ pub struct ModbusCoilData {
     pub data_access_type: Option<DataAccessType>,
     /// (boolean) data value, or map to a bit of a registers
     pub data_value: ModbusCoilDataValueType,
+    /// external program to run after a register is updated
+    pub external_program: Option<String>,
 }
 
 impl ModbusCoilData {
@@ -430,7 +437,7 @@ impl ModbusCoilData {
         }
     }
 
-    pub fn update(&mut self, value: bool, rdb: &mut ModbusRegisterDatabase) {
+    pub fn update(&mut self, value: bool, rdb: &mut ModbusRegisterDatabase, e: EndiannessType) {
         let d = &mut self.data_value;
         match d {
             ModbusCoilDataValueType::Independent(_) => {
@@ -442,7 +449,7 @@ impl ModbusCoilData {
                     .get_mut(&c.register)
                     .expect(&format!("missing register @ {}", c.register));
                 let mut current_values = Vec::<u16>::new();
-                let _ = register.write_to_be_u16(&mut current_values);
+                let _ = register.write_into_u16(&mut current_values, e);
                 let register_idx = (c.bit / 16) as usize;
                 let bit_idx = (c.bit % 16) as usize;
                 if value == true {
@@ -450,12 +457,12 @@ impl ModbusCoilData {
                 } else {
                     current_values[register_idx] = current_values[register_idx] & (!(1 << bit_idx));
                 }
-                register.read_from_be_u16(&mut current_values.iter().peekable());
+                register.read_from_u16(&mut current_values.iter().peekable(), e);
             }
         }
     }
 
-    pub fn read(&self, rdb: &ModbusRegisterDatabase) -> bool {
+    pub fn read(&self, rdb: &ModbusRegisterDatabase, e: EndiannessType) -> bool {
         let d = &self.data_value;
         match d {
             ModbusCoilDataValueType::Independent(IndependentCoil { value }) => *value,
@@ -465,7 +472,7 @@ impl ModbusCoilData {
                     .get(&c.register)
                     .expect(&format!("missing register @ {}", c.register));
                 let mut current_values = Vec::<u16>::new();
-                let _ = register.write_to_be_u16(&mut current_values);
+                let _ = register.write_into_u16(&mut current_values, e);
                 let register_idx = (c.bit / 16) as usize;
                 let bit_idx = (c.bit % 16) as usize;
                 current_values[register_idx] & (1 << bit_idx) != 0
@@ -474,7 +481,7 @@ impl ModbusCoilData {
     }
 }
 
-#[derive(Debug, Clone, Deserialize)]
+#[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct ModbusCoilDatabase {
     db: HashMap<u16, ModbusCoilData>,
 }
@@ -486,6 +493,7 @@ impl ModbusCoilDatabase {
         values: Vec<bool>,
         function_code: FunctionCode,
         rdb: &mut ModbusRegisterDatabase,
+        e: EndiannessType,
     ) -> anyhow::Result<usize, ModbusExceptionCode> {
         let mut value_it = values.iter().peekable();
         let mut total_updated = 0_usize;
@@ -493,7 +501,7 @@ impl ModbusCoilDatabase {
         while let Some(data) = self.db.get_mut(&addr) {
             if data.is_function_code_supported(function_code) {
                 if let Some(new_data) = value_it.next() {
-                    data.update(*new_data, rdb);
+                    data.update(*new_data, rdb, e);
                     total_updated += 1;
                 } else {
                     return Err(ModbusExceptionCode::IllegalDataValue);
@@ -515,6 +523,7 @@ impl ModbusCoilDatabase {
         count: u16,
         function_code: FunctionCode,
         rdb: &ModbusRegisterDatabase,
+        e: EndiannessType,
     ) -> anyhow::Result<Vec<bool>, ModbusExceptionCode> {
         let mut coils = Vec::<bool>::new();
         let mut count = count as usize;
@@ -522,7 +531,7 @@ impl ModbusCoilDatabase {
         //let mut printout = String::new();
         while let Some(data) = self.db.get(&addr) {
             if data.is_function_code_supported(function_code) {
-                coils.push(data.read(rdb));
+                coils.push(data.read(rdb, e));
                 count -= 1;
                 if count == 0 {
                     return Ok(coils);
