@@ -1,8 +1,14 @@
 use clap::ArgEnum;
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 use std::io;
 use tokio_serial::{DataBits, Parity, StopBits};
 
+#[derive(ArgEnum, Clone, Copy, PartialEq, Debug, Deserialize)]
+pub enum EndiannessType {
+    BigEndian,
+    LittleEndian,
+}
+
 #[derive(ArgEnum, Clone, PartialEq, Debug, Deserialize)]
 pub enum DeviceType {
     Client,
@@ -15,7 +21,7 @@ pub enum ProtocolType {
     TCP,
 }
 
-#[derive(ArgEnum, Clone, Copy, PartialEq, Debug, Deserialize)]
+#[derive(ArgEnum, Clone, Copy, PartialEq, Debug, Serialize, Deserialize)]
 pub enum DataType {
     Float32,
     Float64,
@@ -26,7 +32,7 @@ pub enum DataType {
     Int64,
 }
 
-#[derive(ArgEnum, Clone, Debug, Copy, PartialEq, Deserialize)]
+#[derive(ArgEnum, Clone, Debug, Copy, PartialEq, Serialize, Deserialize)]
 pub enum DataModelType {
     DiscreteInputs,
     Coils,
@@ -37,7 +43,7 @@ pub enum DataModelType {
     AllType,
 }
 
-#[derive(ArgEnum, Clone, Debug, Copy, PartialEq, Deserialize)]
+#[derive(ArgEnum, Clone, Debug, Copy, PartialEq, Serialize, Deserialize)]
 pub enum DataAccessType {
     ReadOnly,
     WriteOnly,
@@ -64,6 +70,7 @@ pub enum FunctionCode {
     GetcommEventLog = 0x0C,
     WriteMultipleCoils = 0x0F,
     WriteMultipleRegisters = 0x10,
+    ReadWriteMultipleRegisters = 0x17,
 }
 
 impl FunctionCode {
